# -*- coding: utf-8 -*-

from imio.restapi.services import add

from plone import api

from plone.restapi.deserializer import json_body

from urban.restapi.exceptions import UndefinedPortalType, DefaultFolderManagerNotFoundError

from Products.urban.utils import getLicenceFolder

import json


class AddLicencePost(add.FolderPost):

    portal_type = ''  # to override in subclasses

    def reply(self):
        elements = json_body(self.request)
        result_list = []
        for element in elements["__elements__"]:
            licence = self.initialize_description_field(element)
            licence = self.set_portal_type(licence)
            licence = self.set_creation_place(licence)
            licence = self.set_default_foldermanager(licence)
            licence = self.set_location_uids(licence)
            licence = self.set_events(licence)
            licence = self.set_contacts(licence)
            self.request.set('BODY', json.dumps(licence))
            result_list.append(super(AddLicencePost, self).reply())
        return result_list

    def set_portal_type(self, data):
        """ """
        if self.portal_type:
            data['@type'] = self.portal_type
        return data

    def set_creation_place(self, data):
        """ """
        portal_type = data.get('@type', None)
        if not portal_type:
            raise UndefinedPortalType
        licence_folder = getLicenceFolder(portal_type)
        self.context = licence_folder
        return data

    def set_default_foldermanager(self, data):
        """ """
        if not ('foldermanagers' in data and data['foldermanagers']):
            portal_urban = api.portal.get_tool('portal_urban')
            for licence_config in portal_urban.objectValues('LicenceConfig'):
                if licence_config.id == data.get('@type').lower():
                    default_foldermanagers_uids = [foldermanager.UID()
                                                   for foldermanager in licence_config.getDefault_foldermanager()]
                    data['foldermanagers'] = default_foldermanagers_uids

            if not data['foldermanagers']:
                raise DefaultFolderManagerNotFoundError(["No default foldermanager for this licence type"])
        return data

    def set_location_uids(self, data):
        """ """
        catalog = api.portal.get_tool("uid_catalog")
        results = catalog.searchResults(**{'portal_type': 'Street'})
        if 'workLocations' in data and data['workLocations']:
            for idx, work_location in enumerate(data['workLocations']):
                if not('street' in work_location) and 'street_ins' in work_location:
                    for result in results:
                        if data['workLocations'][idx]['street_ins'] == result.getObject().getStreetCode():
                            data['workLocations'][idx]['street'] = result.getObject().UID()
                            break
                else:
                    zipcode = data['workLocations'][idx]['zipcode'] if 'zipcode' in data['workLocations'][idx] else ""
                    localite = data['workLocations'][idx]['localite'] if 'localite' in data['workLocations'][idx] else ""
                    data['description']['data'] += ("<p>Situation : %s %s %s %s</p>" %
                                                    (
                                                        data['workLocations'][idx]['number'],
                                                        data['workLocations'][idx]['street'],
                                                        zipcode,
                                                        localite
                                                    ))

        if 'businessOldLocation' in data and data['businessOldLocation']:
            for idx, business_old_location in enumerate(data['businessOldLocation']):
                if not ('street' in business_old_location) and 'street_ins' in business_old_location:
                    for result in results:
                        if data['businessOldLocation'][idx]['street_ins'] == result.getObject().getStreetCode():
                            data['businessOldLocation'][idx]['street'] = result.getObject().UID()
                            break
                else:
                    zipcode = data['businessOldLocation'][idx]['zipcode'] \
                        if 'zipcode' in data['businessOldLocation'][idx] else ""
                    localite = data['businessOldLocation'][idx]['localite'] \
                        if 'localite' in data['businessOldLocation'][idx] else ""
                    data['description']['data'] += ("<p>Ancienne adresse de l'exploitation : %s %s %s %s</p>" %
                                                    (
                                                        data['businessOldLocation'][idx]['number'],
                                                        data['businessOldLocation'][idx]['street'],
                                                        zipcode,
                                                        localite
                                                    ))
        return data

    def set_events(self, data):
        """ """
        if '__children__' in data:
            for idx, child in enumerate(data['__children__']):
                if child['@type'] == 'UrbanEvent':
                    if child['event_id'] and 'urbaneventtypes' not in child:
                        data['__children__'][idx]['urbaneventtypes'] = "{0}/portal_urban/{1}/urbaneventtypes/{2}".format(
                            api.portal.getSite().absolute_url(),
                            self.portal_type.lower(),
                            child['event_id']
                    )
        return data

    def set_contacts(self, data):
        """ """
        catalog = api.portal.get_tool('portal_catalog')

        if 'architects' in data and data['architects']:
            architects_args = []
            for idx, architect in enumerate(data['architects']):
                if not ('architect_id' in architect):
                    fullname = "{0} {1}".format(architect['name1'], architect['name2'])
                    if fullname:
                        architects = catalog(portal_type='Architect', Title=fullname)
                        if len(architects) == 1:
                            architects_args.append(architects[0].getObject().absolute_url())
                        else:
                            data['description']['data'] += (u"<p>Architecte : %s %s %s %s %s</p>" % (
                                                                architect.get('name1', ""),
                                                                architect.get('name2', ""),
                                                                architect.get('street', ""),
                                                                architect.get('zipcode', ""),
                                                                architect.get('city', "")
                                                            ))
                    else:
                        print("Empty architect ?")
                else:
                    architects_args.append(architect['architect_id'])

            if architects_args:
                data['architects'] = architects_args

        if 'geometricians' in data and data['geometricians']:
            geometricians_args = []
            for idx, geometrician in enumerate(data['geometricians']):
                if not ('geometrician_id' in geometrician):
                    fullname = "{0} {1}".format(geometrician['name1'], geometrician['name2'])
                    if fullname:
                        geometricians = catalog(portal_type='Geometrician', Title=fullname)
                        if len(geometricians) == 1:
                            geometricians_args.append(geometricians[0].getObject().absolute_url())
                        else:
                            data['description']['data'] += (u"<p>Géomètre : %s %s %s %s %s</p>" %
                                                            (
                                                                geometrician.get('name1', ""),
                                                                geometrician.get('name2', ""),
                                                                geometrician.get('street', ""),
                                                                geometrician.get('zipcode', ""),
                                                                geometrician.get('city', "")
                                                            ))
                    else:
                        print("Empty geometrician ?")
                else:
                    geometricians_args.append(geometrician['geometrician_id'])

            if geometricians_args:
                data['geometricians'] = geometricians_args

        if 'notaries' in data and data['notaries']:
            notaries_args = []
            for idx, notary in enumerate(data['notaries']):
                if not ('notary_id' in notary):
                    fullname = "{0} {1}".format(notary['name1'], notary['name2'])
                    if fullname:
                        notaries = catalog(portal_type='Notary', Title=fullname)
                        if len(notaries) == 1:
                            notaries_args.append(notaries[0].getObject().absolute_url())
                        else:
                            data['description']['data'] += (u"<p>Notaire : %s %s %s %s %s</p>" %
                                                            (
                                                                notary.get('name1', ""),
                                                                notary.get('name2', ""),
                                                                notary.get('street', ""),
                                                                notary.get('zipcode', ""),
                                                                notary.get('city', "")
                                                            ))
                    else:
                        print("Empty notary ?")
                else:
                    notaries_args.append(notary['notary_id'])

            if notaries_args:
                data['notaries'] = notaries_args

        return data

    @staticmethod
    def initialize_description_field(data):
        if 'description' not in data:
            data['description'] = {}
        if 'data' not in data['description']:
            data['description']['data'] = ""
        if 'content-type' not in data['description']:
            data['description']['content-type'] = "text/html"

        return data



# -*- coding: utf-8 -*-

from urban.restapi.services.content.licence import base


class AddBuildLicencePost(base.AddLicencePost):

    portal_type = 'BuildLicence'

    def reply(self):
        result = super(AddBuildLicencePost, self).reply()
        return result



# -*- coding: utf-8 -*-

from plone.restapi.exceptions import APIError


class UrbanRestAPIError(APIError):
    """ Base class for urban rest api errors"""


class UndefinedPortalType(UrbanRestAPIError):
    """
    Raised when no portal_type defined.
    """


class EnvironmentRubricNotFound(UrbanRestAPIError):
    """
    Raised when no corresponding environment rubric is found
    in urban.
    """


class DefaultFolderManagerNotFoundError(UrbanRestAPIError):
    """
    Raised when no default foldermanager for this licence type is found
    in urban.
    """



# -*- coding: utf-8 -*-

from plone.restapi.services import Service
from Products.urban.browser.searchparcelsview import SearchParcelsView


class SearchParcel(Service):

    portal_type = ""  # to override in subclasses

    def reply(self):
        """
        End point to search cadastral parcel
        callable with 'GET' and '@parcel'
        Must add minimum one query parameters from this list :
            - division
            - section
            - radical
            - bis
            - exposant
            - puissance
            - location
            - street_number
            - parcel_owner

        return an object with a list of serialized parcel object and
            the count of item return
        """
        search = SearchParcelsView(self.context, self.request)
        result = search.search_parcels()

        return {"items": self._serialize_parcel(result), "items_total": len(result)}

    def _serialize_parcel(self, parcels):
        serialized_parcels = []
        for parcel in parcels:
            attributes = self._get_attributes(parcel)
            serialized_parcels.append(
                {attr: getattr(parcel, attr) for attr in attributes}
            )
        return serialized_parcels

    def _get_attributes(self, parcel):
        attributes = dir(parcel)
        return [
            attr
            for attr in attributes
            if not attr.startswith("_") and not callable(getattr(parcel, attr))
        ]



# -*- coding: utf-8 -*-

from urban.restapi.services.content.licence import base

import json

from urban.restapi.services.content.utils import set_rubrics


class AddLicencePost(base.AddLicencePost):

    def reply(self):
        data = json_body(self.request)
        for licence in data['__children__']:
            new_data = set_rubrics(licence)
            self.request.set('BODY', json.dumps(new_data))
            result = super(AddLicencePost, self).reply()
        return result



# -*- coding: utf-8 -*-
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.restapi.testing import PLONE_RESTAPI_AT_FUNCTIONAL_TESTING
from plone.testing import z2

import urban.restapi


class UrbanRestapiLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        import plone.app.dexterity
        self.loadZCML(package=plone.app.dexterity)
        self.loadZCML(package=urban.restapi)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'urban.restapi:default')


URBAN_RESTAPI_FIXTURE = UrbanRestapiLayer()


URBAN_RESTAPI_INTEGRATION_TESTING = IntegrationTesting(
    bases=(URBAN_RESTAPI_FIXTURE,),
    name='UrbanRestapiLayer:IntegrationTesting',
)


URBAN_RESTAPI_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(URBAN_RESTAPI_FIXTURE, PLONE_RESTAPI_AT_FUNCTIONAL_TESTING),
    name='UrbanRestapiLayer:FunctionalTesting',
)


URBAN_RESTAPI_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        URBAN_RESTAPI_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE,
    ),
    name='UrbanRestapiLayer:AcceptanceTesting',
)



# -*- coding: utf-8 -*-

import ast
import base64
import datetime
import json

from plone import api
from plone.restapi.deserializer import json_body
from urban.restapi.services.content.esb import base
from urban.restapi.services.content.utils import set_rubrics, set_location


class AddEsbEnvClassThreePost(base.AddLicencePost):

    portal_type = 'EnvClassThree'

    def reply(self):
        body = json_body(self.request)
        archive_title = body['formName']
        json_data = body['json']
        pdf_data = body['pdf']
        attachment_archive = body['attachmentArchive']

        licence = self.get_envclassthree_dict()
        if pdf_data:
            attachment_dict = self.get_attachment_dict()
            attachment_dict['file']['data'] = pdf_data + "==="
            attachment_dict['file']['filename'] = "Permis.pdf"
            attachment_dict['title'] = "Pièce jointe principale"
            attachment_dict['file']['content-type'] = "application/pdf"
            licence['__children__'].append(attachment_dict)

        if attachment_archive:
            attachment_dict = self.get_attachment_dict()
            attachment_dict['file']['data'] = attachment_archive['data'] + "==="
            attachment_dict['file']['filename'] = "{}.{}".format("pieces_jointes_formulaire",
                                                                 attachment_archive['archiveType'])
            attachment_dict['title'] = archive_title
            attachment_dict['file']['content-type'] = attachment_archive['archiveMimeType']
            licence['__children__'].append(attachment_dict)

        if json_data:
            envclass3_json_file = base64.b64decode(json_data + "===")
            envclass3_json = json.loads(envclass3_json_file)['dataStore']
            if "etablissement" in envclass3_json:
                if 'description' in envclass3_json['etablissement']:
                    licence['licenceSubject'] = envclass3_json['etablissement']['description']
                if 'rubriques' in envclass3_json['etablissement']:
                    rubrics_list = []
                    if 'numRubrique' in envclass3_json['etablissement']['rubriques']['item']:
                        rubrics_list.append(envclass3_json['etablissement']['rubriques']['item']['numRubrique'])
                    else:
                        for item in envclass3_json['etablissement']['rubriques']['item']:
                            rubrics_list.append(item['numRubrique'])
                    licence['rubrics'] = rubrics_list
                    set_rubrics(self, licence)
                if 'adresse' in envclass3_json['etablissement']:
                    worklocation_dict = self.get_work_locations_dict()
                    worklocation_dict['zipcode'] = str(envclass3_json['etablissement']['adresse']['cp'])
                    worklocation_dict['localite'] = envclass3_json['etablissement']['adresse']['localite']
                    if 'rue' in envclass3_json['etablissement']['adresse']:
                        worklocation_dict['street'] = envclass3_json['etablissement']['adresse']['rue']
                    if 'numero' in envclass3_json['etablissement']['adresse']:
                        worklocation_dict['number'] = str(envclass3_json['etablissement']['adresse']['numero'])
                    if 'boite' in envclass3_json['etablissement']['adresse']:
                        worklocation_dict['number'] += ' {}'.format(str(envclass3_json['etablissement']['adresse']
                                                                        ['boite']))
                    set_location(self, worklocation_dict, licence)

                if 'natura2000' in envclass3_json['etablissement']:
                    if 'site' in envclass3_json['etablissement']['natura2000'] and envclass3_json['etablissement']['natura2000']['site']:
                        details = ""
                        if 'ug' in envclass3_json['etablissement']['natura2000'] and envclass3_json['etablissement']['natura2000']['ug']:
                            details = envclass3_json['etablissement']['natura2000']['ug']
                        licence['description'] += (u"<p>Natura2000 : Site : {} / détails : {}</p>".format(
                                                       envclass3_json['etablissement']['natura2000']['site'],
                                                       details
                                                   ))
                        licence['natura2000'] = 'True'
                if 'natura2000Details' in envclass3_json['etablissement']:
                    licence['natura2000Details'] = (u"<p>Natura2000Details : {}</p>".format(
                                                   envclass3_json['etablissement']['natura2000Details']
                                               ))
            if "demandeur" in envclass3_json:
                applicant_dict = self.get_applicant_dict()
                if "identification" in envclass3_json["demandeur"]:
                    if "nom" in envclass3_json["demandeur"]["identification"]:
                        applicant_dict['name2'] = envclass3_json["demandeur"]["identification"]['nom']
                    if "prenom" in envclass3_json["demandeur"]["identification"]:
                        applicant_dict['name1'] = envclass3_json["demandeur"]["identification"]['prenom']
                    if "nature" in envclass3_json["demandeur"]["identification"]:
                        if envclass3_json["demandeur"]["identification"]['nature'] == "personneMorale":
                            applicant_dict['@type'] = "Corporation"
                    if "civilite" in envclass3_json["demandeur"]["identification"]:
                        if envclass3_json["demandeur"]["identification"]['civilite'] == "monsieur":
                            applicant_dict['personTitle'] = "mister"
                        elif envclass3_json["demandeur"]["identification"]['civilite'] == "madame":
                            applicant_dict['personTitle'] = "madam"
                if "telecom" in envclass3_json["demandeur"]:
                    if 'tel' in envclass3_json["demandeur"]["telecom"]:
                        applicant_dict['phone'] = envclass3_json["demandeur"]["telecom"]['tel']
                    if 'tel2' in envclass3_json["demandeur"]["telecom"]:
                        applicant_dict['gsm'] = envclass3_json["demandeur"]["telecom"]['tel2']
                    if envclass3_json["demandeur"]["telecom"]['mail']:
                        applicant_dict['email'] = envclass3_json["demandeur"]["telecom"]['mail']
                if "adresse" in envclass3_json["demandeur"]:
                    if 'rue' in envclass3_json["demandeur"]["adresse"]:
                        applicant_dict['street'] = envclass3_json["demandeur"]["adresse"]['rue']
                    if 'numero' in envclass3_json["demandeur"]["adresse"]:
                        applicant_dict['number'] = str(envclass3_json["demandeur"]["adresse"]['numero'])
                    if 'boite' in envclass3_json["demandeur"]["adresse"]:
                        applicant_dict['street'] = "{} {}".format(applicant_dict['street'],
                                                                  str(envclass3_json["demandeur"]["adresse"]['boite']))
                    if 'cp' in envclass3_json["demandeur"]["adresse"]:
                        applicant_dict['zipcode'] = str(envclass3_json["demandeur"]["adresse"]['cp'])
                    if 'localite' in envclass3_json["demandeur"]["adresse"]:
                        applicant_dict['city'] = envclass3_json["demandeur"]["adresse"]['localite']
                    if 'pays' in envclass3_json["demandeur"]["adresse"]:
                        if envclass3_json["demandeur"]["adresse"]['pays'] != "BE":
                            applicant_dict['localite'] = "{} {}".format(applicant_dict['localite'],
                                                                        envclass3_json["demandeur"]["adresse"]['pays'])
                licence['__children__'].append(applicant_dict)

            if "situation" in envclass3_json:
                if "parcelles" in envclass3_json["situation"]:
                    if "item" in envclass3_json["situation"]["parcelles"]:
                        portionout_list = []
                        for item in envclass3_json["situation"]["parcelles"]['item']:
                            portionout_dict = self.get_parcel_dict()
                            portionout_dict['division'] = str(item["ins"]["codeDivision"])
                            portionout_dict['section'] = item["ref"]["section"]
                            portionout_dict['radical'] = str(int(item["ref"]["numero"]))
                            portionout_dict['bis'] = str(item["ref"]["indice"]) if item["ref"]["indice"] != '00' else ''
                            portionout_dict['exposant'] = item["ref"]["exposant"]
                            portionout_dict['puissance'] = str(int(item["ref"]["diviseur"])) if item["ref"]["diviseur"] != '000' else ''
                            # Always true from official RW portal form
                            portionout_dict['is_official'] = 'True'
                            portionout_list.append(portionout_dict)
                            licence['__children__'].append(portionout_dict)

        # Auto-insert deposit event with current date
        deposit_event_dict = self.get_deposit_event_dict()
        deposit_event_dict['eventDate'] = datetime.datetime.today().strftime('%d/%m/%Y')
        portal_urban = api.portal.get_tool('portal_urban')
        found = False
        for licence_config in portal_urban.objectValues('LicenceConfig'):
            if licence_config.id == 'envclassthree':
                eventtype_folder = licence_config.urbaneventtypes
                for event in eventtype_folder.objectValues():
                    # Looking for the first Deposit event
                    if "Products.urban.interfaces.IDepositEvent" in event.getEventTypeType():
                        deposit_event_dict['event_id'] = event.id
                        deposit_event_dict['title'] = "Dépôt de la demande par le formulaire de la Région Wallonne"
                        deposit_event_dict['urbaneventtypes'] = event.absolute_url()
                        licence['__children__'].append(deposit_event_dict)
                        found = True
                        break
            if found:
                break

        self.request.set('BODY', json.dumps(licence))
        result = super(AddEsbEnvClassThreePost, self).reply()
        # wso2 don't like @
        result = ast.literal_eval(repr(result).replace('@', ''))
        return result

    def get_envclassthree_dict(self):
        return {
            "@type": self.portal_type,
            'portalType': self.portal_type,
            'referenceDGATLP': '',
            'licenceSubject': '',
            'review_state': '',
            'description': '',
            'natura2000': '',
            'natura2000Details': '',
            'workLocations': [],
            '__children__': [],
        }

    def get_attachment_dict(self):
        return {
            "@type": "File",
            "title": '',
            "creators": [
                "ESB"
            ],
            "description": "This is a file",
            "file": {
                "data": "",
                "encoding": "base64",
                "filename": "",
                "content-type": ""
            }
        }

    def get_applicant_dict(self):
        return {
            '@type': 'Applicant',
            'personTitle': '',
            'name1': '',
            'name2': '',
            'email': '',
            'phone': '',
            'gsm': '',
            'fax': '',
            'street': '',
            'zipcode': '',
            'city': '',
            'country': '',
        }

    def get_parcel_dict(self):
        return {
            '@type': 'PortionOut',
            'complete_name': '',
            'outdated': '',
            'division': '',
            'section': '',
            'radical': '',
            'bis': '',
            'exposant': '',
            'puissance': '',
            'is_official': ''
        }

    def get_work_locations_dict(self):
        return {
            'number': '',
            'street': '',
            'zipcode': '',
            'localite': '',
        }

    def get_deposit_event_dict(self):
        return {
            '@type': 'UrbanEvent',
            'event_id': '',
            'type': '',
            'eventDate': '',
            'decisionDate': '',
            'decision': '',
        }



# -*- coding: utf-8 -*-

from plone.restapi.deserializer import json_body
from urban.restapi.services.content.licence import environment_base

import json


class AddEnvClassThreePost(environment_base.AddLicencePost):

    portal_type = 'EnvClassThree'

    def reply(self):
        result = super(AddEnvClassThreePost, self).reply()
        return result



# -*- coding: utf-8 -*-

import json

from Products.CMFPlone.utils import normalizeString
from imio.restapi.services import add
from plone import api
from plone.api.exc import InvalidParameterError
from plone.restapi.deserializer import json_body
from urban.restapi.exceptions import UndefinedPortalType, DefaultFolderManagerNotFoundError, EnvironmentRubricNotFound
from urban.restapi.services.content.utils import quote_parenthesis
from Products.urban.utils import getLicenceFolder


class AddElementPost(add.FolderPost):

    portal_type = ''  # to override in subclasses

    def reply(self):
        elements = json_body(self.request)
        for element in elements["__elements__"]:
            element = self.set_portal_type(element)
            licence = self.initialize_description_field(element)
            licence = self.set_creation_place(licence)
            licence = self.set_default_foldermanager(licence)
            licence = self.set_location_uids(licence)
            licence = self.set_events(licence)
            licence = self.set_rubrics(licence)
            licence = self.set_contacts(licence)
            self.request.set('BODY', json.dumps(licence))
            result = super(AddElementPost, self).reply()
            # post-treatment
            self.set_status(licence, result)
            self.set_location_number_format(licence, result)
        return result

    def set_portal_type(self, data):
        """ """
        if self.portal_type:
            data['@type'] = self.portal_type
        else:
            data['@type'] = data['portalType']
        return data

    def set_creation_place(self, data):
        """ """
        portal_type = data.get('@type', None)
        if not portal_type:
            raise UndefinedPortalType
        licence_folder = getLicenceFolder(portal_type)
        self.context = licence_folder
        return data

    def set_default_foldermanager(self, data):
        """ """
        if not ('foldermanagers' in data and data['foldermanagers']):
            portal_urban = api.portal.get_tool('portal_urban')
            for licence_config in portal_urban.objectValues('LicenceConfig'):
                if licence_config.id == data.get('@type').lower():
                    default_foldermanagers_uids = [foldermanager.UID()
                                                   for foldermanager in licence_config.getDefault_foldermanager()]
                    data['foldermanagers'] = default_foldermanagers_uids

            if not data['foldermanagers']:
                raise DefaultFolderManagerNotFoundError(["No default foldermanager for this licence type"])
        return data

    def set_location_uids(self, data):
        """ """
        catalog = api.portal.get_tool("uid_catalog")
        results = catalog.searchResults(**{'portal_type': 'Street'})
        if 'workLocations' in data and data['workLocations']:
            for idx, work_location in enumerate(data['workLocations']):
                if data['workLocations'][idx]['number']:
                    data['workLocations'][idx]['number'] = str(data['workLocations'][idx]['number'])
                if 'bestaddress_key' in work_location and work_location['bestaddress_key']:
                    for result in results:
                        if int(data['workLocations'][idx]['bestaddress_key']) == result.getObject().getBestAddressKey():
                            data['workLocations'][idx]['street'] = str(result.getObject().UID())

                            break
                else:
                    zipcode = data['workLocations'][idx]['zipcode'] if 'zipcode' in data['workLocations'][idx] else ""
                    localite = data['workLocations'][idx]['locality'] if 'locality' in data['workLocations'][idx] else ""
                    data['description']['data'] += ("<p>Situation : %s %s %s %s</p>" %
                                                    (
                                                        data['workLocations'][idx]['number'],
                                                        data['workLocations'][idx]['street'],
                                                        zipcode,
                                                        localite
                                                    ))

        if 'businessOldLocation' in data and data['businessOldLocation']:
            for idx, business_old_location in enumerate(data['businessOldLocation']):
                if 'bestaddress_key' in business_old_location and business_old_location['bestaddress_key']:
                    for result in results:
                        if int(data['business_old_location'][idx]['bestaddress_key']) == result.getObject().getBestAddressKey():
                            data['businessOldLocation'][idx]['street'] = result.getObject().UID()
                            break
                else:
                    zipcode = data['businessOldLocation'][idx]['zipcode'] \
                        if 'zipcode' in data['businessOldLocation'][idx] else ""
                    localite = data['businessOldLocation'][idx]['locality'] \
                        if 'locality' in data['businessOldLocation'][idx] else ""
                    data['description']['data'] += ("<p>Ancienne adresse de l'exploitation : %s %s %s %s</p>" %
                                                    (
                                                        data['businessOldLocation'][idx]['number'],
                                                        data['businessOldLocation'][idx]['street'],
                                                        zipcode,
                                                        localite
                                                    ))
        return data

    def set_events(self, data):
        """ """
        if '__children__' in data:
            for idx, child in enumerate(data['__children__']):
                if child['@type'] == 'UrbanEvent':
                    if child['event_id'] and 'urbaneventtypes' not in child:
                        data['__children__'][idx]['urbaneventtypes'] = "{0}/portal_urban/{1}/urbaneventtypes/{2}".format(
                            api.portal.getSite().absolute_url(),
                            data['@type'].lower(),
                            child['event_id']
                    )
        return data

    def set_contacts(self, data):
        """ """
        catalog = api.portal.get_tool('portal_catalog')
        if 'architects' in data and data['architects']:
            architects_args = []
            for idx, architect in enumerate(data['architects']):
                if not ('architect_id' in architect):
                    fullname = u"{0} {1}".format(architect['name1'], architect['name2'])
                    if fullname:
                        architects = catalog(portal_type='Architect', Title=quote_parenthesis(fullname).replace("-", " "))
                        if len(architects) == 1:
                            architects_args.append(architects[0].getObject().absolute_url())
                        elif len(architects) == 0 and architect['force_create'] == 'True':
                            container = api.content.get(path='/urban/architects')
                            normalized_id = normalizeString(u"{}_{}".format(architect.get('name1', ""), architect.get('name2', "")))
                            object_id = container.invokeFactory('Architect',
                                                                id= normalized_id,
                                                                name1=architect.get('name1', ""),
                                                                name2=architect.get('name2', ""),
                                                                phone=architect.get('phone', ""),
                                                                gsm=architect.get('gsm', ""),
                                                                email=architect.get('email', ""),
                                                                personTitle=architect.get('personTitle', ""),
                                                                street=architect.get('street', ""),
                                                                zipcode=str(architect.get('zipcode', "")),
                                                                city=architect.get('city', ""))
                            architects = catalog(portal_type='Architect', id=object_id)
                            architects_args.append(architects[0].getObject().absolute_url())
                        else:
                            data['description']['data'] += (u"<p>Architecte : %s %s %s %s %s</p>" % (
                                                                architect.get('name1', ""),
                                                                architect.get('name2', ""),
                                                                architect.get('street', ""),
                                                                architect.get('zipcode', ""),
                                                                architect.get('city', "")
                                                            ))
                    else:
                        print("Empty architect ?")
                else:
                    architects_args.append(architect['architect_id'])

            data['architects'] = architects_args

        if 'geometricians' in data and data['geometricians']:
            geometricians_args = []
            for idx, geometrician in enumerate(data['geometricians']):
                if not ('geometrician_id' in geometrician):
                    fullname = u"{0} {1}".format(geometrician['name1'], geometrician['name2'])
                    if fullname:
                        geometricians = catalog(portal_type='Geometrician', Title=quote_parenthesis(fullname))
                        if len(geometricians) == 1:
                            geometricians_args.append(geometricians[0].getObject().absolute_url())
                        elif len(geometricians) == 0 and geometrician['force_create'] == 'True':
                            container = api.content.get(path='/urban/geometricians')
                            normalized_id = normalizeString(u"{}_{}".format(geometrician.get('name1', ""), geometrician.get('name2', "")))
                            object_id = container.invokeFactory('Geometrician',
                                                                id= normalized_id,
                                                                name1=geometrician.get('name1', ""),
                                                                name2=geometrician.get('name2', ""),
                                                                phone=geometrician.get('phone', ""),
                                                                gsm=geometrician.get('gsm', ""),
                                                                email=geometrician.get('email', ""),
                                                                personTitle=geometrician.get('personTitle', ""),
                                                                street=geometrician.get('street', ""),
                                                                zipcode=str(geometrician.get('zipcode', "")),
                                                                city=geometrician.get('city', ""))
                            geometricians = catalog(portal_type='Geometrician', id=object_id)
                            geometricians_args.append(geometricians[0].getObject().absolute_url())
                        elif len(geometricians) > 1 and geometrician['force_create'] == 'True':
                            fullname_default = u"Géomètre par défaut"
                            geometricians = catalog(portal_type='Geometrician', Title=fullname_default)
                            if len(geometricians) == 1:
                                geometricians_args.append(geometricians[0].getObject().absolute_url())
                                data['description']['data'] += (u"<p>Trop de résultats pour le géomètre %s, un géomètre par défaut a été lié</p>" %
                                                                (
                                                                    fullname
                                                                ))
                            else:
                                print("Pas de géomètre défini par défaut : {}".format(fullname_default))
                        else:
                            data['description']['data'] += (u"<p>Géomètre : %s %s %s %s %s</p>" %
                                                            (
                                                                geometrician.get('name1', ""),
                                                                geometrician.get('name2', ""),
                                                                geometrician.get('street', ""),
                                                                str(geometrician.get('zipcode', "")),
                                                                geometrician.get('city', "")
                                                            ))
                    else:
                        print("Empty geometrician ?")

                else:
                    geometricians_args.append(geometrician['geometrician_id'])

            data['geometricians'] = geometricians_args

        if 'notaries' in data and data['notaries']:
            notaries_args = []
            for idx, notary in enumerate(data['notaries']):
                if not ('notary_id' in notary):
                    fullname = u"{0} {1}".format(notary['name1'], notary['name2'])
                    if fullname:
                        notaries = catalog(portal_type='Notary', Title=quote_parenthesis(fullname).replace("-", " "))
                        if len(notaries) == 1:
                            notaries_args.append(notaries[0].getObject().absolute_url())
                        elif len(notaries) == 0 and notary['force_create'] == 'True':
                            container = api.content.get(path='/urban/notaries')
                            normalized_id = normalizeString(u"{}_{}".format(notary.get('name1', ""), notary.get('name2', "")))
                            object_id = container.invokeFactory('Notary',
                                                                id= normalized_id,
                                                                name1=notary.get('name1', ""),
                                                                name2=notary.get('name2', ""),
                                                                phone=notary.get('phone', ""),
                                                                gsm=notary.get('gsm', ""),
                                                                email=notary.get('email', ""),
                                                                personTitle=notary.get('personTitle', ""),
                                                                street=notary.get('street', ""),
                                                                zipcode=str(notary.get('zipcode', "")),
                                                                city=notary.get('city', ""))
                            notaries = catalog(portal_type='Notary', id=object_id)
                            notaries_args.append(notaries[0].getObject().absolute_url())
                        else:
                            data['description']['data'] += (u"<p>Notaire : %s %s %s %s %s</p>" %
                                                            (
                                                                notary.get('name1', ""),
                                                                notary.get('name2', ""),
                                                                notary.get('street', ""),
                                                                notary.get('zipcode', ""),
                                                                notary.get('city', "")
                                                            ))
                    else:
                        print("Empty notary ?")
                else:
                    notaries_args.append(notary['notary_id'])

            data['notaryContact'] = notaries_args

        return data

    def set_status(self, licence, result):
        if 'wf_state' in licence and licence['wf_state']:
            site = api.portal.getSite()
            licence_object = site.reference_catalog.lookupObject(result['UID'])
            try:
                api.content.transition(licence_object, licence['wf_state'])
            except InvalidParameterError:
                print("***no valid state***")

        if 'wf_transition' in licence and licence['wf_transition']:
            site = api.portal.getSite()
            licence_object = site.reference_catalog.lookupObject(result['UID'])
            try:
                api.content.transition(obj=licence_object, to_state=licence['wf_transition'])
            except InvalidParameterError:
                print("***no valid transition***")
                # api.content.transition(licence_object, 'nonapplicable')
        return licence

    def set_location_number_format(self, licence, result):
        if 'workLocations' in licence and licence['workLocations']:
            site = api.portal.getSite()
            licence_object = site.reference_catalog.lookupObject(result['UID'])
            wl = licence_object.getWorkLocations()
            wll = list(wl)
            new_wll = []
            for wl in wll:
                if wl['number'] and isinstance(wl['number'], unicode):
                    wl['number'] = wl['number'].encode("utf-8")
                new_wll.append(wl)
            licence_object.setWorkLocations(tuple(new_wll))

    def set_rubrics(self, data):
        """ """
        rubrics_args = data.get(u'rubrics', '')
        data[u'rubrics'] = []

        if rubrics_args:
            catalog = api.portal.get_tool('portal_catalog')
            rubric_uids = []
            for rubric in rubrics_args:
                rubric_brains = catalog(id=rubric, portal_type='EnvironmentRubricTerm')
                if len(rubric_brains) != 1:
                    print(u"***Environment Rubric Not Found or too many results: {} ***".format(rubric.encode("ascii", "ignore")))
                else:
                    rubric_uids.append(rubric_brains[0].UID)
            data[u'rubrics'] = rubric_uids
        return data


    @staticmethod
    def initialize_description_field(data):
        if 'description' not in data:
            data['description'] = {}
        if 'data' not in data['description']:
            data['description']['data'] = ""
        if 'content-type' not in data['description']:
            data['description']['content-type'] = "text/html"

        return data



# -*- coding: utf-8 -*-
"""Installer for the urban.restapi package."""

from setuptools import find_packages
from setuptools import setup


long_description = '\n\n'.join([
    open('README.rst').read(),
    open('CONTRIBUTORS.rst').read(),
    open('CHANGES.rst').read(),
])


setup(
    name='urban.restapi',
    version='1.0a1',
    description="specific urban restapi for interoperability",
    long_description=long_description,
    # Get more from https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.3",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords='Python Plone',
    author='Julien Jaumotte',
    author_email='julien.jaumotte@imio.be',
    url='https://pypi.python.org/pypi/urban.restapi',
    license='GPL version 2',
    packages=find_packages('src', exclude=['ez_setup']),
    namespace_packages=['urban'],
    package_dir={'': 'src'},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        # -*- Extra requirements: -*-
        'plone.api',
        'imio.restapi',
        'Products.GenericSetup',
        'setuptools',
        'z3c.jbot',
    ],
    extras_require={
        'test': [
            'plone.app.testing',
            # Plone KGS does not use this version, because it would break
            # Remove if your package shall be part of coredev.
            # plone_coredev tests as of 2016-04-01.
            'plone.testing',
            'imio.restapi[test]',
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    [console_scripts]
    update_locale = urban.restapi.locales.update:update_locale
    """,
)



# -*- coding: utf-8 -*-

from eea.faceted.vocabularies.autocomplete import IAutocompleteSuggest
from plone.restapi.services import Service
from zope.component import getMultiAdapter


class SearchAdress(Service):

    portal_type = ""  # to override in subclasses

    def reply(self):
        """
        End point to search street
        callable with 'GET' and '@address'
        Must add minimum one character as query parameters

        return an object with a list of object compose of uid and name of the street and
            the count of item return
        """
        adapter = getMultiAdapter(
            (self.context, self.request),
            IAutocompleteSuggest,
            name="sreets-autocomplete-suggest",
        )

        items = [
            {"name": street["label"], "uid": street["value"]}
            for street in adapter.compute_suggestions()
        ]

        return {"items": items, "items_total": len(items)}



# -*- coding: utf-8 -*-

from urban.restapi.services.content.licence import base


class AddCodtParcelOutLicencePost(base.AddLicencePost):

    portal_type = 'CODT_ParcelOutLicence'

    def reply(self):
        result = super(AddCodtParcelOutLicencePost, self).reply()
        return result



# -*- coding: utf-8 -*-

from imio.restapi.services import add
from plone import api

from plone.restapi.deserializer import json_body
from plone.restapi.services.content import get


import json

from Products.urban.interfaces import IGenericLicence
from urban.restapi.services.content.utils import set_creation_place, set_default_foldermanager


class AddLicencePost(add.FolderPost):

    portal_type = ''  # to override in subclasses

    def reply(self):
        licence = json_body(self.request)
        licence = set_creation_place(self, licence)
        licence = set_default_foldermanager(self, licence)
        self.request.set('BODY', json.dumps(licence))
        result = super(AddLicencePost, self).reply()
        return result


class LicenceStatus(get.ContentGet):

    portal_type = ''

    def reply(self):
        reference = self.request.ref
        catalog = api.portal.get_tool('portal_catalog')
        brains = catalog(object_provides='Products.urban.interfaces.IEnvClassThree')
        results = []
        for brain in brains:
            if brain.getObject().getReference() == reference:
                results.append(brain.getObject())
        if len(results) == 1:
            workflow_tool = api.portal.get_tool('portal_workflow')
            workflow_state = workflow_tool.getStatusOf('urban_licence_workflow', results[0])
            response = {'review_state': workflow_state['review_state'],
                        'date': str(workflow_state['time'].Date())}
            return response
        else:
            return {'error': 'too many or no result(s)'}



# -*- coding: utf-8 -*-

import time

from urban.restapi.exceptions import UndefinedPortalType, DefaultFolderManagerNotFoundError, EnvironmentRubricNotFound
from plone import api
from Products.urban.utils import getLicenceFolder


magic_dict = {
    "\x1f\x8b\x08": "gz",
    "\x75\x73\x74\x61\x72": "tar",
    "\x50\x4b\x03\x04": "zip"
    }
magic_max_len = max(len(x) for x in magic_dict)


def set_creation_place(context, data):
    """ """
    portal_type = data.get('@type', None)
    if not portal_type:
        raise UndefinedPortalType
    licence_folder = getLicenceFolder(portal_type)
    context.context = licence_folder
    return data


def set_default_foldermanager(context, data):
    """ """
    if not ('foldermanagers' in data and data['foldermanagers']):
        portal_urban = api.portal.get_tool('portal_urban')
        for licence_config in portal_urban.objectValues('LicenceConfig'):
            if licence_config.id == data.get('@type').lower():
                default_foldermanagers_uids = [foldermanager.UID()
                                               for foldermanager in licence_config.getDefault_foldermanager()]
                data['foldermanagers'] = default_foldermanagers_uids

        if not data['foldermanagers']:
            raise DefaultFolderManagerNotFoundError(["No default foldermanager for this licence type"])
    return data


def set_rubrics(context, data):
    """ """
    rubrics_args = data.get(u'rubrics', '')
    data[u'rubrics'] = []

    if rubrics_args:
        catalog = api.portal.get_tool('portal_catalog')
        rubric_uids = []
        for rubric in rubrics_args:
            rubric_brains = catalog(id=rubric)
            if len(rubric_brains) != 1:
                raise EnvironmentRubricNotFound(rubric)
            rubric_uids.append(rubric_brains[0].UID)
        data[u'rubrics'] = rubric_uids
        return data


def set_location(context, data, licence):
    """ """
    catalog = api.portal.get_tool("portal_catalog")
    if 'street' in data:
        results = catalog(portal_type='Street', Title=str(data['street'].encode('UTF-8')))
        if len(results) == 1:
            data['street'] = results[0].getObject().UID()
            licence['workLocations'].append(data)
        else:
            licence['description'] += ("<p>Situation : %s %s %s %s</p>" %
                                       (
                                        str(data['number']),
                                        data['street'],
                                        str(data['zipcode']),
                                        data['localite']
                                       ))


def file_type(filename):
    with open(filename) as f:
        file_start = f.read(magic_max_len)
    for magic, filetype in magic_dict.items():
        if file_start.startswith(magic):
            return filetype
    return "no match"


def quote_parenthesis(s):
    # We need to quote parentheses when searching text indices
    if '(' in s:
        s = s.replace('(', '"("')
    if ')' in s:
        s = s.replace(')', '")"')
    return s


def benchmark_decorator(method):
    def replacement(self, *args, **kwargs):
        # if not self.benchmarking:
        #     return method(self, *args, **kwargs)
        if not self._benchmark.get(method.__name__):
            self._benchmark[method.__name__] = {'counter': 0, 'elapsed_time': 0}
        self._benchmark[method.__name__]['counter'] += 1
        start_time = time.time()
        returned_value = method(self, *args, **kwargs)
        self._benchmark[method.__name__]['elapsed_time'] += time.time() - start_time
        self._benchmark[method.__name__]['average_time'] = \
            self._benchmark[method.__name__]['elapsed_time'] / self._benchmark[method.__name__]['counter']
        return returned_value

    return replacement
